@@ -75,6 +75,9 @@ typedef struct
 	int column;
 	EelPreferencesGroupPopulateFunction populate_function;
 	const char *enumeration_list_unique_exceptions;
+	const char *accessible_description;
+	int n_accessible_descriptions;
+	const char **accessible_description_array;
 } EelPreferencesItemDescription;
 
 typedef struct
